"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getV2ArchivedSlidesRecord = exports.getV2SlidesRecord = exports.getV2ChaptersRecord = exports.getV2StoryboardRecord = exports.getChaptersFromNestedSlides = exports.getChapterSlidesById = exports.getStructureV1ArchivedSlides = exports.getChapterCustomFields = exports.getSlideCustomFields = exports.normalizeStructureV1Chapters = exports.normalizeStructureV1Subslides = exports.flatChapterContent = exports.getChapterSlidesFromNested = exports.getNormalizedChapterEntity = exports.getNormalizedSlideEntity = exports.normalizeEntityForStructureV1 = exports.nestedSlideHasNoSubslides = exports.isSubslide = exports.getFiltersList = exports.getDefinitelyArchivedSlides = exports.listIncludesSlide = exports.getHiddenSlides = exports.getNormalizedChapters = exports.getSlidesWithSubslides = exports.getAllContent = exports.normalizeChapter = exports.normalizeSlide = exports.normalizeEntity = void 0;
const interfaces_1 = require("../types/interfaces");
const types_1 = require("../types/types");
function normalizeEntity(entity, entityType) {
    if (typeof entity === 'string') {
        return {
            [entityType]: entity,
            hidden: false
        };
    }
    return entity;
}
exports.normalizeEntity = normalizeEntity;
function normalizeSlide(entity) {
    return normalizeEntity(entity, types_1.EntityType.Slide);
}
exports.normalizeSlide = normalizeSlide;
function normalizeChapter(entity) {
    return normalizeEntity(entity, types_1.EntityType.Chapter);
}
exports.normalizeChapter = normalizeChapter;
function getAllContent(structure) {
    const chaptersId = Object.keys(structure.chapters);
    const chaptersContent = chaptersId
        .flatMap(chapterId => structure.chapters[chapterId].content.map(entity => normalizeSlide(entity)));
    return chaptersContent;
}
exports.getAllContent = getAllContent;
function getSlidesWithSubslides(structure) {
    return getAllContent(structure).filter(slide => normalizeSlide(slide).subslides);
}
exports.getSlidesWithSubslides = getSlidesWithSubslides;
function getNormalizedChapters(structure) {
    const chaptersId = Object.keys(structure.chapters);
    return chaptersId.map(chapterId => {
        return [
            chapterId,
            structure.chapters[chapterId].content
                .map(slideEntity => normalizeSlide(slideEntity))
        ];
    });
}
exports.getNormalizedChapters = getNormalizedChapters;
function getHiddenSlides(structure) {
    const slides = getAllContent(structure);
    const hiddenSlides = slides.filter(slide => slide.hidden);
    hiddenSlides.push(...(getSlidesWithSubslides(structure)
        .flatMap(slide => slide.subslides)
        .flatMap((subslide) => normalizeSlide(subslide))
        .filter(subslide => subslide.hidden)));
    return hiddenSlides;
}
exports.getHiddenSlides = getHiddenSlides;
function listIncludesSlide(list, slideId) {
    return list.some(slideEntity => (slideEntity.slide === slideId));
}
exports.listIncludesSlide = listIncludesSlide;
function getDefinitelyArchivedSlides(structure) {
    return structure.archivedSlides ?
        structure.archivedSlides.map(slideEntity => normalizeSlide(slideEntity)) :
        [];
}
exports.getDefinitelyArchivedSlides = getDefinitelyArchivedSlides;
function getFiltersList(types) {
    if (types) {
        const activeFilters = Object.entries(types).filter((filter) => filter[1]);
        return activeFilters.map((filter) => filter[0]);
    }
    return;
}
exports.getFiltersList = getFiltersList;
function isSubslide(nestedSlidesData, slideId) {
    const subslidesList = getSubslidesList(nestedSlidesData);
    return subslidesList.includes(slideId);
}
exports.isSubslide = isSubslide;
function getSubslidesList(nestedSlidesData) {
    return nestedSlidesData
        .filter(nestedSlide => nestedSlide.subslides.length)
        .flatMap(nestedSlide => nestedSlide.subslides)
        .map(subslide => subslide.id);
}
function nestedSlideHasNoSubslides(nestedSlide) {
    return !nestedSlide.subslides || !nestedSlide.subslides.length;
}
exports.nestedSlideHasNoSubslides = nestedSlideHasNoSubslides;
function normalizeEntityForStructureV1(entity) {
    if (entity.type === interfaces_1.StructureApi.SlideType.Hidden) {
        return `!${entity.id}`;
    }
    return entity.id;
}
exports.normalizeEntityForStructureV1 = normalizeEntityForStructureV1;
function getNormalizedSlideEntity(entity) {
    if (isV1HiddenEntity(entity)) {
        return {
            slide: entity.replace('!', ''),
            hidden: true
        };
    }
    return entity;
}
exports.getNormalizedSlideEntity = getNormalizedSlideEntity;
function getNormalizedChapterEntity(entity) {
    if (isV1HiddenEntity(entity)) {
        return {
            chapter: entity.replace('!', ''),
            hidden: true
        };
    }
    return entity;
}
exports.getNormalizedChapterEntity = getNormalizedChapterEntity;
function getChapterSlidesFromNested(nestedSlidesWithoutSubslides, chapterData) {
    return nestedSlidesWithoutSubslides
        .filter(nestedSlide => nestedSlide.chapter?.id === chapterData.id)
        .map(nestedSlide => {
        return nestedSlide.type === interfaces_1.StructureApi.SlideType.Archived ? `!${nestedSlide.id}` : nestedSlide.id;
    });
}
exports.getChapterSlidesFromNested = getChapterSlidesFromNested;
function flatChapterContent(chapterContent) {
    return chapterContent.flatMap(slideEntity => {
        if (typeof slideEntity !== 'string' && slideEntity.slide) {
            const slide = slideEntity.slide;
            const subslides = slideEntity.subslides;
            return subslides ? [slide, ...subslides] : [slide];
        }
        return slideEntity;
    });
}
exports.flatChapterContent = flatChapterContent;
function normalizeStructureV1Subslides(structureJson) {
    const slides = structureJson.slides;
    const rawSlides = Object.entries(slides).map(slideEntry => {
        const slideData = slideEntry[1];
        if (slideData.subslides) {
            slideData.subslides = slideData.subslides
                .filter(slide => !isV1HiddenEntity(slide))
                .map(subslideName => {
                if (typeof subslideName === 'string' && subslideIsHidden(structureJson, subslideName)) {
                    return {
                        slide: subslideName.replace('!', ''),
                        hidden: true
                    };
                }
                if (typeof subslideName === 'string') {
                    return getNormalizedSlideEntity(subslideName);
                }
                return subslideName;
            });
        }
        return [slideEntry[0], slideData];
    });
    return Object.fromEntries(rawSlides);
}
exports.normalizeStructureV1Subslides = normalizeStructureV1Subslides;
function normalizeStructureV1Chapters(structure) {
    const chapters = structure.chapters;
    const rawChapters = Object.entries(chapters).map(chapterEntry => {
        const { content } = chapterEntry[1];
        const isHiddenChapter = (id) => {
            const slideChapter = getV1SlideChapter(structure, id) ?? getV1SlideChapter(structure, `!${id}`) ?? '';
            return getV1ChapterType(structure, slideChapter) === interfaces_1.StructureApi.ChapterType.Hidden;
        };
        const normalizedContent = content
            .filter(slideName => !isV1HiddenEntity(slideName))
            .map(slideName => isHiddenChapter(slideName) ? { slide: slideName, hidden: true } : slideName);
        const normalizedChapterData = {
            name: chapterEntry[1].name,
            content: normalizedContent,
            ...getChapterCustomFields(structure, chapterEntry[0])
        };
        return [chapterEntry[0], normalizedChapterData];
    });
    return Object.fromEntries(rawChapters);
}
exports.normalizeStructureV1Chapters = normalizeStructureV1Chapters;
function getSlideCustomFields(structure, slideId) {
    const slide = structure.slides[slideId];
    if (!slide) {
        return {};
    }
    const { name, template, ...customFields } = slide;
    return customFields || {};
}
exports.getSlideCustomFields = getSlideCustomFields;
function getChapterCustomFields(structure, chapterId) {
    const chapter = structure.chapters[chapterId];
    if (!chapter) {
        return {};
    }
    const { name, content, ...customFields } = chapter;
    return customFields || {};
}
exports.getChapterCustomFields = getChapterCustomFields;
function getStructureV1ArchivedSlides(structureJson) {
    const slidesV1Subslides = Object.values(structureJson.slides).flatMap(slide => slide.subslides || []).filter(elem => elem);
    const chaptersV1Slides = Object.values(structureJson.chapters).flatMap(chapterData => chapterData.content);
    const allExistingV1Slides = [...slidesV1Subslides, ...chaptersV1Slides];
    Object.keys(structureJson.slides).forEach(slide => {
        if (allExistingV1Slides.includes(slide) || allExistingV1Slides.includes(`!${slide}`)) {
            return;
        }
        allExistingV1Slides.push(slide);
    });
    return allExistingV1Slides
        .map(slideId => ({ id: slideId, type: getV1SlideType(structureJson, slideId) }))
        .filter(slideData => slideData.type === interfaces_1.StructureApi.SlideType.Archived)
        .map(slideData => ({ slide: slideData.id.replace('!', ''), chapter: getV1SlideChapter(structureJson, slideData.id) }));
}
exports.getStructureV1ArchivedSlides = getStructureV1ArchivedSlides;
function getV1ChapterType(structureJson, chapterId) {
    const chapterIncludedToStoryboard = structureJson.storyboard.includes(chapterId) || structureJson.storyboard.includes(`!${chapterId}`);
    const chapterIsHidden = structureJson.storyboard.some(chapter => chapter === `!${chapterId}`);
    const isHidden = !chapterIncludedToStoryboard || isV1HiddenEntity(chapterId) || chapterIsHidden;
    return isHidden ? interfaces_1.StructureApi.ChapterType.Hidden : interfaces_1.StructureApi.ChapterType.Visible;
}
function getV1SlideType(structureJson, slideId) {
    const slideChapter = getV1SlideChapter(structureJson, slideId);
    const parentSlide = getV1ParentSlide(structureJson, slideId, slideChapter || '');
    if (parentSlide && !isV1HiddenEntity(slideId)) {
        return getV1SlideType(structureJson, parentSlide);
    }
    if (isV1HiddenEntity(slideId) || !slideChapter) {
        return interfaces_1.StructureApi.SlideType.Archived;
    }
    if (slideChapter) {
        return getV1ChapterType(structureJson, slideChapter);
    }
    return interfaces_1.StructureApi.SlideType.Visible;
}
function getV1SlideChapter(structureJson, slideId) {
    const chapterSlidesSet = {};
    const slidesSubslidesSet = {};
    Object.keys(structureJson.chapters).forEach(chapterId => {
        chapterSlidesSet[chapterId] = structureJson.chapters[chapterId].content;
    });
    const chapterEntry = findSlide(Object.entries(chapterSlidesSet), slideId);
    if (chapterEntry) {
        return chapterEntry[0];
    }
    Object.keys(structureJson.slides).forEach(slideId => {
        slidesSubslidesSet[slideId] = structureJson.slides[slideId].subslides || [];
    });
    const parentSlide = findSlide(Object.entries(slidesSubslidesSet), slideId);
    if (parentSlide) {
        const chapterEntry = Object.entries(chapterSlidesSet)
            .find(([chapterId, slides]) => slides.includes(parentSlide[0]) || slides.includes(`!${parentSlide[0]}`));
        return chapterEntry ? chapterEntry[0] : undefined;
    }
    return;
}
function getV1ParentSlide(structureJson, slideId, slideChapter) {
    const parent = Object.entries(structureJson.slides).find(([slide, slideData]) => {
        return slideData.subslides?.includes(slideId) || slideData.subslides?.includes(`!${slideId}`);
    });
    const parentId = parent ? parent[0] : undefined;
    return structureJson.chapters[slideChapter]?.content.find(slide => slide === parentId || slide === `!${parentId}`);
}
function findSlide(entries, slideId) {
    return entries.find(([slide, subslides]) => subslides.includes(slideId) || subslides.includes(`!${slideId}`));
}
function getChapterSlidesById(nestedSlides, chapterId) {
    const chapterSlides = nestedSlides.filter(slide => slide.chapter?.id === chapterId).map(nestedSlide => {
        if (nestedSlide.type === interfaces_1.StructureApi.SlideType.Hidden) {
            return { slide: nestedSlide.id, hidden: true };
        }
        return nestedSlide.id;
    });
    return chapterSlides.map(slideEntity => {
        if (typeof slideEntity === 'string') {
            const nestedSlide = nestedSlides.find(slide => slide.id === slideEntity);
            if (nestedSlide?.subslides.length) {
                return {
                    slide: slideEntity,
                    subslides: nestedSlide?.subslides.map(subslide => ({
                        slide: subslide.id,
                        hidden: subslide.type === interfaces_1.StructureApi.SlideType.Hidden
                    }))
                };
            }
            return slideEntity;
        }
        const nestedSlide = nestedSlides.find(slide => slide.id === slideEntity.slide);
        const slideData = {
            slide: slideEntity.slide,
            hidden: slideEntity.hidden,
        };
        if (nestedSlide?.subslides?.length) {
            slideData.subslides = nestedSlide?.subslides.map(subslide => ({
                slide: subslide.id,
                hidden: subslide.type === interfaces_1.StructureApi.SlideType.Hidden
            }));
        }
        return slideData;
    });
}
exports.getChapterSlidesById = getChapterSlidesById;
function getChaptersFromNestedSlides(nestedSlidesData) {
    return nestedSlidesData
        .map(slideData => ({
        id: slideData.chapter?.id,
        name: slideData.chapter?.name,
        type: slideData.chapter?.type,
        storyboardIndex: slideData.chapter?.storyboardIndex,
        custom: slideData.chapter?.custom,
    }))
        .filter(chapterData => !!chapterData.id)
        .reduce((acc, chapterData) => {
        if (!acc.some(elem => elem.id === chapterData?.id)) {
            acc.push(chapterData);
        }
        return acc;
    }, []);
}
exports.getChaptersFromNestedSlides = getChaptersFromNestedSlides;
function getV2StoryboardRecord(typedChapters) {
    const chaptersForStoryboard = typedChapters.filter(chapter => chapter.storyboardIndex !== undefined)
        .sort((first, second) => {
        if (first.storyboardIndex === undefined || second.storyboardIndex === undefined) {
            return 0;
        }
        return first.storyboardIndex - second.storyboardIndex;
    });
    const storyboard = chaptersForStoryboard.map(chapterData => chapterData.type === interfaces_1.StructureApi.ChapterType.Hidden ? { chapter: chapterData.id, hidden: true } : chapterData.id);
    return { storyboard };
}
exports.getV2StoryboardRecord = getV2StoryboardRecord;
function getV2ChaptersRecord(typedChapters, nestedSlidesData) {
    const chapterEntries = typedChapters.map(chapterData => [
        chapterData.id,
        {
            name: chapterData.name,
            content: getChapterSlidesById(nestedSlidesData, chapterData.id || ''),
            ...chapterData.custom,
        }
    ]);
    return { chapters: Object.fromEntries(chapterEntries) };
}
exports.getV2ChaptersRecord = getV2ChaptersRecord;
function getV2SlidesRecord(nestedSlidesData) {
    const slidesAsEntries = nestedSlidesData.map(slideData => [slideData.id, { name: slideData.name, template: slideData.template, ...slideData.custom }]);
    const slidesRecord = { slides: Object.fromEntries(slidesAsEntries) };
    const subslidesAsEntries = nestedSlidesData
        .flatMap(slideData => slideData.subslides)
        .map(slideData => [slideData.id, { name: slideData.name, template: slideData.template, ...slideData.custom }]);
    subslidesAsEntries.forEach(subslideEntry => slidesRecord.slides[subslideEntry[0]] = subslideEntry[1]);
    return slidesRecord;
}
exports.getV2SlidesRecord = getV2SlidesRecord;
function getV2ArchivedSlidesRecord(nestedSlidesData) {
    return {
        archivedSlides: nestedSlidesData
            .filter(slideData => slideData.type === interfaces_1.StructureApi.SlideType.Archived)
            .map(slideData => ({ slide: slideData.id, chapter: slideData.chapter?.id }))
    };
}
exports.getV2ArchivedSlidesRecord = getV2ArchivedSlidesRecord;
function isV1HiddenEntity(entity) {
    return entity.startsWith('!');
}
function subslideIsHidden(structureJson, subslideName) {
    const subslideChapter = getV1SlideChapter(structureJson, subslideName);
    const subslideParent = getV1ParentSlide(structureJson, subslideName, subslideChapter ?? '');
    const parentSlideType = getV1SlideType(structureJson, subslideParent ?? '');
    const subslideChapterType = getV1ChapterType(structureJson, subslideChapter ?? '');
    return typeof subslideName === 'string' && (parentSlideType === interfaces_1.StructureApi.SlideType.Hidden || subslideChapterType === interfaces_1.StructureApi.ChapterType.Hidden);
}
//# sourceMappingURL=index.js.map